import { AfterViewInit, Component } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { DescriptionTemplateTypeService } from '@app/core/services/description-template-type/description-template-type.service';
import { SnackBarNotificationLevel, UiNotificationService } from '@app/core/services/notification/ui-notification-service';
import { BaseComponent } from '@common/base/base.component';
import { FormService } from '@common/forms/form-service';
import { TranslateService } from '@ngx-translate/core';
import { takeUntil } from 'rxjs/operators';

@Component({
  selector: 'app-description-type-editor',
  templateUrl: './description-type-editor.component.html',
  styleUrls: ['./description-type-editor.component.scss']
})
export class DescriptionTypeEditorComponent extends BaseComponent implements AfterViewInit {

	formGroup: FormGroup = null;
	descriptionTypeModel: DescriptionTypeEditorModel

	constructor(
		private descriptionTemplateTypeService: DescriptionTemplateTypeService,
		private formService: FormService,
		private uiNotificationService: UiNotificationService,
		private language: TranslateService,
		private router: Router
	) { 
		super();
	}

	ngAfterViewInit(): void {
		this.descriptionTypeModel = new DescriptionTypeEditorModel();
		setTimeout(() => {
			this.formGroup = this.descriptionTypeModel.buildForm();
		});
	}

  	formSubmit(): void {
		this.formService.touchAllFormFields(this.formGroup);
		if (!this.isFormValid()) { return; }
		this.onSubmit();
	}

	public isFormValid() {
		return this.formGroup.valid;
	}

  	onSubmit(): void {
		console.log(this.formGroup.value);
		this.descriptionTemplateTypeService.createType(this.formGroup.value)
			.pipe(takeUntil(this._destroyed))
			.subscribe(
				complete => this.onCallbackSuccess(),
				error => this.onCallbackError(error)
			);
	}

	onCallbackSuccess(): void {
		this.uiNotificationService.snackBarNotification(this.language.instant('GENERAL.SNACK-BAR.SUCCESSFUL-CREATION'), SnackBarNotificationLevel.Success);
		this.router.navigate(['/description-types']);
	}

	onCallbackError(errorResponse: any) {
		// this.setErrorModel(errorResponse.error);
		// this.formService.validateAllFormFields(this.formGroup);
	}

  public cancel(): void {
		this.router.navigate(['/description-types']);
	}

}

export class DescriptionTypeEditorModel {
  	public id: string;
  	public name: string;

	buildForm(): FormGroup {
		const formGroup = new FormBuilder().group({
			id: [this.id],
			name: [this.name, Validators.required],
		});
		return formGroup;
	}
}
